package org.netkernel.ext.system.representation.impl;

import java.util.*;

import org.netkernel.ext.system.representation.IRepDeployedModules;
import org.netkernel.layer0.meta.IPrototypeMeta;
import org.netkernel.layer0.meta.ISpaceElements;
import org.netkernel.layer0.meta.ISpaceMeta;
import org.netkernel.urii.IEndpoint;
import org.netkernel.urii.IIdentifier;
import org.netkernel.urii.IMetaRepresentation;
import org.netkernel.util.MultiMap;

public class ProtoMap
{
	private final Map<Class,String> mPrototypesForClass;
	private final Map<String,String[]> mInstancesOfProto;
	private static final String[] sNoInstances=new String[0];
	
	public ProtoMap(IRepDeployedModules aDeployedModules)
	{
		
		Map<Class,String> prototypesForClass = new HashMap<Class,String>();
		Object[] spaceHashes=aDeployedModules.getHierarchy().getValues("/modules/module//spaces/space/hash");
		
		//find all prototypes
		for (Object h: spaceHashes)
		{	String spaceHash=(String)h;
			IMetaRepresentation meta=aDeployedModules.metadataForHash(spaceHash);
			ISpaceMeta sm=(ISpaceMeta)meta;
			ISpaceElements elements=sm.getElements();
			for (int i=0; i<elements.size(); i++)
			{	if (elements.getDelegatedSpace(i)==null)
				{	IIdentifier id=elements.getIdentifier(i);
					if (id!=null)
					{	String elementHash=spaceHash+"/"+id.toString();
						IMetaRepresentation em=aDeployedModules.metadataForHash(elementHash);
						if (em instanceof IPrototypeMeta)
						{	IPrototypeMeta pm=(IPrototypeMeta)em;
							prototypesForClass.put(pm.getImplementation(),elementHash);
						}
					}
				}
			}
		}
		mPrototypesForClass=prototypesForClass;
		
		//find all instances
		MultiMap instances =new MultiMap(128, 4);
		IEndpoint lastPhysicalEp=null;
		for (Object h: spaceHashes)
		{	String spaceHash=(String)h;
			IMetaRepresentation meta=aDeployedModules.metadataForHash(spaceHash);
			ISpaceMeta sm=(ISpaceMeta)meta;
			ISpaceElements elements=sm.getElements();
			for (int i=0; i<elements.size(); i++)
			{		
				IEndpoint physicalEp=elements.getPhysicalEndpoint(i);
				if (physicalEp!=lastPhysicalEp)
				{	
					Class impl=physicalEp.getClass();
					if (prototypesForClass.containsKey(impl))
					{	String hash=aDeployedModules.hashForComponent(physicalEp);
						instances.put(impl, hash);
					}
				}
				lastPhysicalEp=physicalEp;
			}
		}
		
		mInstancesOfProto = new HashMap<String,String[]>();
		for (Map.Entry<Class, String> entry : prototypesForClass.entrySet())
		{	String hash=entry.getValue();
			List eps=instances.get(entry.getKey());
			if (eps.size()>0)
			{	String[] epArray = new String[eps.size()];
				eps.toArray(epArray);
				mInstancesOfProto.put(hash, epArray);
			}
		}
		
	}
	
	public String getPrototypeHashFor(Class aEndpointClass)
	{	return mPrototypesForClass.get(aEndpointClass);
	}
	
	public String[] getEndpointInstancesOf(String aPrototypeHash)
	{	String[] result=mInstancesOfProto.get(aPrototypeHash);
		if (result==null)
		{	result = sNoInstances;
		}
		return result;
	}
}
