package org.netkernel.ext.system.representation.impl;

import java.util.*;
import org.netkernel.ext.system.representation.IRepDeployedModules;
import org.netkernel.layer0.meta.*;
import org.netkernel.urii.IIdentifier;
import org.netkernel.urii.IMetaRepresentation;
import org.netkernel.util.MultiMap;

public class RepresentationReferences
{
	private final Map<String,String[]> mInstancesOfRep;
	
	public RepresentationReferences(IRepDeployedModules aModules) throws Exception
	{
		Object[] spaceHashes=aModules.getHierarchy().getValues("/modules/module//spaces/space/hash");
		/*
		//find all representations
		List<String> reps=new ArrayList<String>();
		for (Object h: spaceHashes)
		{	String spaceHash=(String)h;
			IMetaRepresentation meta=aModules.metadataForHash(spaceHash);
			ISpaceMeta sm=(ISpaceMeta)meta;
			ISpaceElements elements=sm.getElements();
			for (int i=0; i<elements.size(); i++)
			{	if (elements.getDelegatedSpace(i)==null)
				{	IIdentifier id=elements.getIdentifier(i);
					if (id!=null)
					{	String elementHash=spaceHash+"/"+id.toString();
						IMetaRepresentation em=aModules.metadataForHash(elementHash);
						if (em instanceof IRepresentationMeta)
						{	reps.add(elementHash);
						}
					}
				}
			}
		}
		*/
		
		//find all logical endpoints with references
		MultiMap instances =new MultiMap(128, 4);
		for (Object h: spaceHashes)
		{	String spaceHash=(String)h;
			IMetaRepresentation meta=aModules.metadataForHash(spaceHash);
			ISpaceMeta sm=(ISpaceMeta)meta;
			ISpaceElements elements=sm.getElements();
			for (int i=0; i<elements.size(); i++)
			{	if (elements.getDelegatedSpace(i)==null)
				{	IIdentifier id=elements.getIdentifier(i);
					if (id!=null)
					{	String elementHash=spaceHash+"/"+id.toString();
						IMetaRepresentation em=aModules.metadataForHash(elementHash);
						if (em instanceof IEndpointMeta)
						{	IEndpointMeta epm = (IEndpointMeta)em;
							Class[] reps=epm.getResponseClasses();
							if (reps!=null)
							{	for (Class c : reps)
								{	String repHash=aModules.hashForClass(c);
									if (repHash!=null)
									{	instances.put(repHash, elementHash);
									}
								}
							}
							for (IEndpointArgumentMeta epam : epm.getArguments())
							{	reps=epam.getRequiredRepresentations();
								if (reps!=null)
								{	for (Class c : reps)
									{	String repHash=aModules.hashForClass(c);
										if (repHash!=null)
										{	instances.put(repHash, elementHash);
										}
									}
								}
							}
						}
					}
				}
			}
		}		
		
		mInstancesOfRep = new HashMap<String,String[]>();
		for (Iterator i=instances.keyIterator(); i.hasNext(); )
		{	String repHash=(String)i.next();
			List eps=instances.get(repHash);
			if (eps.size()>0)
			{	String[] epArray = new String[eps.size()];
				eps.toArray(epArray);
				mInstancesOfRep.put(repHash, epArray);
			}
		}
	}
	
	public String[] getReferencesTo(String aRepresentationHash)
	{	return mInstancesOfRep.get(aRepresentationHash);
	}
}